@@ -39,6 +39,7 @@ pub const DEFAULT_TPU_COALESCE_MS: u64 = 5;
 pub struct Tpu {
     fetch_stage: FetchStage,
     sigverify_stage: SigVerifyStage,
+    vote_sigverify_stage: SigVerifyStage,
     banking_stage: BankingStage,
     cluster_info_vote_listener: ClusterInfoVoteListener,
     broadcast_stage: BroadcastStage,
@@ -53,6 +54,7 @@ impl Tpu {
         retransmit_slots_receiver: RetransmitSlotsReceiver,
         transactions_sockets: Vec<UdpSocket>,
         tpu_forwards_sockets: Vec<UdpSocket>,
+        tpu_vote_sockets: Vec<UdpSocket>,
         broadcast_sockets: Vec<UdpSocket>,
         subscriptions: &Arc<RpcSubscriptions>,
         transaction_status_sender: Option<TransactionStatusSender>,
@@ -71,11 +73,14 @@ impl Tpu {
         cluster_confirmed_slot_sender: GossipDuplicateConfirmedSlotsSender,
     ) -> Self {
         let (packet_sender, packet_receiver) = channel();
+        let (vote_packet_sender, vote_packet_receiver) = channel();
         let fetch_stage = FetchStage::new_with_sender(
             transactions_sockets,
             tpu_forwards_sockets,
+            tpu_vote_sockets,
             exit,
             &packet_sender,
+            &vote_packet_sender,
             poh_recorder,
             tpu_coalesce_ms,
         );
@@ -86,14 +91,26 @@ impl Tpu {
             SigVerifyStage::new(packet_receiver, verified_sender, verifier)
         };
 
-        let (verified_vote_packets_sender, verified_vote_packets_receiver) = unbounded();
+        let (verified_tpu_vote_packets_sender, verified_tpu_vote_packets_receiver) = unbounded();
+
+        let vote_sigverify_stage = {
+            let verifier = TransactionSigVerifier::new_reject_non_vote();
+            SigVerifyStage::new(
+                vote_packet_receiver,
+                verified_tpu_vote_packets_sender,
+                verifier,
+            )
+        };
+
+        let (verified_gossip_vote_packets_sender, verified_gossip_vote_packets_receiver) =
+            unbounded();
         let cluster_info_vote_listener = ClusterInfoVoteListener::new(
             exit,
             cluster_info.clone(),
-            verified_vote_packets_sender,
+            verified_gossip_vote_packets_sender,
             poh_recorder,
             vote_tracker,
-            bank_forks,
+            bank_forks.clone(),
             subscriptions.clone(),
             verified_vote_sender,
             gossip_verified_vote_hash_sender,
@@ -107,7 +124,8 @@ impl Tpu {
             cluster_info,
             poh_recorder,
             verified_receiver,
-            verified_vote_packets_receiver,
+            verified_tpu_vote_packets_receiver,
+            verified_gossip_vote_packets_receiver,
             transaction_status_sender,
             replay_vote_sender,
         );
@@ -119,12 +137,14 @@ impl Tpu {
             retransmit_slots_receiver,
             exit,
             blockstore,
+            &bank_forks,
             shred_version,
         );
 
         Self {
             fetch_stage,
             sigverify_stage,
+            vote_sigverify_stage,
             banking_stage,
             cluster_info_vote_listener,
             broadcast_stage,
@@ -135,6 +155,7 @@ impl Tpu {
         let results = vec![
             self.fetch_stage.join(),
             self.sigverify_stage.join(),
+            self.vote_sigverify_stage.join(),
             self.cluster_info_vote_listener.join(),
             self.banking_stage.join(),
         ];
